@@ -295,6 +295,7 @@ class NotebookClient(LoggingConfigurable):
         super().__init__(**kw)
         self.nb: NotebookNode = nb
         self.km: t.Optional[KernelManager] = km
+        self.owns_km: bool = km is None  # whether the NotebookClient owns the kernel manager
         self.kc: t.Optional[KernelClient] = None
         self.reset_execution_trackers()
         self.widget_registry: t.Dict[str, t.Dict] = {
@@ -321,13 +322,13 @@ class NotebookClient(LoggingConfigurable):
         # our front-end mimicing Output widgets
         self.comm_objects: t.Dict[str, t.Any] = {}
 
-    def start_kernel_manager(self) -> KernelManager:
+    def create_kernel_manager(self) -> KernelManager:
         """Creates a new kernel manager.
 
         Returns
         -------
-        kc : KernelClient
-            Kernel client as created by the kernel manager ``km``.
+        km : KernelManager
+            Kernel manager whose client class is asynchronous.
         """
         if not self.kernel_name:
             kn = self.nb.metadata.get('kernelspec', {}).get('name')
@@ -362,8 +363,8 @@ class NotebookClient(LoggingConfigurable):
 
     _cleanup_kernel = run_sync(_async_cleanup_kernel)
 
-    async def async_start_new_kernel_client(self, **kwargs) -> t.Tuple[KernelClient, str]:
-        """Creates a new kernel client.
+    async def async_start_new_kernel(self, **kwargs) -> None:
+        """Creates a new kernel.
 
         Parameters
         ----------
@@ -371,42 +372,34 @@ class NotebookClient(LoggingConfigurable):
             Any options for ``self.kernel_manager_class.start_kernel()``. Because
             that defaults to AsyncKernelManager, this will likely include options
             accepted by ``AsyncKernelManager.start_kernel()``, which includes ``cwd``.
-
-        Returns
-        -------
-        kc : KernelClient
-            Kernel client as created by the kernel manager ``km``.
-        kernel_id : string-ized version 4 uuid
-           The id of the started kernel.
         """
         assert self.km is not None
         resource_path = self.resources.get('metadata', {}).get('path') or None
         if resource_path and 'cwd' not in kwargs:
             kwargs["cwd"] = resource_path
 
-        # if self.km is a MultiKernelManager, it doesn't have an ipykernel attribute
-        # so no extra_arguments can be passed
-        if hasattr(self.km, 'ipykernel') and self.km.ipykernel and self.ipython_hist_file:
+        has_history_manager_arg = any(
+            arg.startswith('--HistoryManager.hist_file') for arg in self.extra_arguments)
+        if (hasattr(self.km, 'ipykernel')
+                and self.km.ipykernel
+                and self.ipython_hist_file
+                and not has_history_manager_arg):
             self.extra_arguments += ['--HistoryManager.hist_file={}'.format(self.ipython_hist_file)]
 
-        kernel_id = await ensure_async(self.km.start_kernel(extra_arguments=self.extra_arguments,
-                                       **kwargs))
+        await ensure_async(self.km.start_kernel(extra_arguments=self.extra_arguments, **kwargs))
 
-        # if self.km is not a KernelManager, it's probably a MultiKernelManager
-        try:
-            self.km.client
-            km = self.km
-        except AttributeError:
-            try:
-                km = self.km.get_kernel(kernel_id)
-            except AttributeError:
-                raise AttributeError('self.km={} has no client() or get_kernel() method, '
-                                     'what is this?'.format(self.km))
-            # since self.km is a MultiKernelManager, it might not be async
-            # so let's ensure the client is async
-            km.client_class = 'jupyter_client.asynchronous.AsyncKernelClient'
+    start_new_kernel = run_sync(async_start_new_kernel)
 
-        self.kc = km.client()
+    async def async_start_new_kernel_client(self) -> KernelClient:
+        """Creates a new kernel client.
+
+        Returns
+        -------
+        kc : KernelClient
+            Kernel client as created by the kernel manager ``km``.
+        """
+        assert self.km is not None
+        self.kc = self.km.client()
         await ensure_async(self.kc.start_channels())
         try:
             await ensure_async(self.kc.wait_for_ready(timeout=self.startup_timeout))
@@ -414,7 +407,7 @@ class NotebookClient(LoggingConfigurable):
             await self._async_cleanup_kernel()
             raise
         self.kc.allow_stdin = False
-        return self.kc, kernel_id
+        return self.kc
 
     start_new_kernel_client = run_sync(async_start_new_kernel_client)
 
@@ -428,14 +421,17 @@ class NotebookClient(LoggingConfigurable):
         When control returns from the yield it stops the client's zmq channels, and shuts
         down the kernel.
         """
-        cleanup_kc = kwargs.pop('cleanup_kc', True)
+        # by default, cleanup the kernel client if we own the kernel manager
+        # and keep it alive if we don't
+        cleanup_kc = kwargs.pop('cleanup_kc', self.owns_km)
 
         # Can't use run_until_complete on an asynccontextmanager function :(
         if self.km is None:
-            self.km = self.start_kernel_manager()
+            self.km = self.create_kernel_manager()
 
         if not self.km.has_kernel:
-            self.start_new_kernel_client(**kwargs)
+            self.start_new_kernel(**kwargs)
+            self.start_new_kernel_client()
         try:
             yield
         finally:
@@ -454,9 +450,11 @@ class NotebookClient(LoggingConfigurable):
 
         Handlers for SIGINT and SIGTERM are also added to cleanup in case of unexpected shutdown.
         """
-        cleanup_kc = kwargs.pop('cleanup_kc', True)
+        # by default, cleanup the kernel client if we own the kernel manager
+        # and keep it alive if we don't
+        cleanup_kc = kwargs.pop('cleanup_kc', self.owns_km)
         if self.km is None:
-            self.km = self.start_kernel_manager()
+            self.km = self.create_kernel_manager()
 
         # self._cleanup_kernel uses run_async, which ensures the ioloop is running again.
         # This is necessary as the ioloop has stopped once atexit fires.
@@ -476,7 +474,8 @@ class NotebookClient(LoggingConfigurable):
             pass
 
         if not self.km.has_kernel:
-            await self.async_start_new_kernel_client(**kwargs)
+            await self.async_start_new_kernel(**kwargs)
+            await self.async_start_new_kernel_client()
         try:
             yield
         finally:
@@ -513,7 +512,7 @@ class NotebookClient(LoggingConfigurable):
         nb : NotebookNode
             The executed notebook.
         """
-        if reset_kc and self.km:
+        if reset_kc and self.owns_km:
             await self._async_cleanup_kernel()
         self.reset_execution_trackers()
 
@@ -529,7 +528,13 @@ class NotebookClient(LoggingConfigurable):
             msg_id = await ensure_async(self.kc.kernel_info())
             info_msg = await self.async_wait_for_reply(msg_id)
             if info_msg is not None:
-                self.nb.metadata['language_info'] = info_msg['content']['language_info']
+                if 'language_info' in info_msg['content']:
+                    self.nb.metadata['language_info'] = info_msg['content']['language_info']
+                else:
+                    raise RuntimeError(
+                        'Kernel info received message content has no "language_info" key.'
+                        'Content is:\n' + str(info_msg['content'])
+                    )
             self.set_widgets_metadata()
 
         return self.nb
